#ifndef CPLXUTILS_H
#define CPLXUTILS_H

#include <complex>

namespace gips
{

    // FUTURE: put this in a namespace (a la physics)
    // FUTURE - move this to a general utils file in common
    template <typename T>
    struct ComplexAbs {
            T operator() (const std::complex<T>& x) { return static_cast<T>( std::abs(x) ); }
    };
    template <class VC, class VR>
    void extractComplexAbs(const VC& cplxin, VR& realout)
    {
        assert (cplxin.size() == realout.size());
        std::transform(cplxin.begin(), cplxin.end(), realout.begin(),
                        ComplexAbs<typename VR::value_type>());
    }
    template <typename T>
    struct ComplexPhase {
            T operator() (const std::complex<T>& x) { return static_cast<T>( std::arg(x) ); }
    };
    template <class VC, class VR>
    void extractComplexPhase(const VC& cplxin, VR& realout)
    {
        assert (cplxin.size() == realout.size());
        std::transform(cplxin.begin(), cplxin.end(), realout.begin(),
                        ComplexPhase<typename VR::value_type>());
    }

    // oobVal template functions return a value that can be used throughout
    // GIPS as an out-of-bound value for that type (generally a NaN
    // for types that support it)
    // default template function
    template <typename T>
    T oobVal()
    {
        static const T n(nan(""));
        return n;
    }
    
    // template function specialization for complex<double>: return complex with
    // real and imag components set to NaN
    template <>
    inline std::complex<double> oobVal<std::complex<double> >()
    {
        static const std::complex<double> n(nan(""), nan(""));
        return n;
    }

    // template function specialization for complex<int>: return complex with 
    // real and imag components set to max int value
    template <>
    inline std::complex<int> oobVal<std::complex<int> >()
    {
        static const std::complex<int> n(std::numeric_limits<int>::max(),
                                         std::numeric_limits<int>::max());
        return n;
    }
    
    // template function specialization for double: return NaN
    template <>
    inline double oobVal<double>()
    {
        static const double n(nan(""));
        return n;
    }

} // namespace
#endif

