// Test for Resample.hxx
// basic compile might look as follows:
// g++ -o ResampleLivenessTest ResampleTest.cc -I/home/maciek/opt/include -I/home/rayg/opt/boost/include -I../Common -L/home/maciek/opt/lib -lfftw3
#include <iostream>
#include <boost/array.hpp>

#include "Resample.hxx"
#include "Physics.h"

enum {SPECTRUM_SIZE = 1025, UPSAMPLE_SIZE = 262144};
typedef boost::array<double, SPECTRUM_SIZE> Data;
typedef gips::SpectralResampling<Data, Data, UPSAMPLE_SIZE> Resampler;

void livenessTest()
{
    gips::physics::WnumScale<double, SPECTRUM_SIZE> 
        wn(5.868544600938967e+02, 1.173708920187793e+03);
    Resampler::settings_t settings(wn);
            
    Resampler::ancillary_t ancil;
    ancil.laserWavenumberRatio = 0.999;

    Data testin, testout;
    std::fill(testin.begin(), testin.end(), 1.0);
    std::fill(testout.begin(), testout.end(), 0.0);

    Resampler resample(settings, Resampler::Ports(&ancil, &testin, &testout));
    for(int i = 0; i < 100; ++i)
    {
        std::cout << "(" << i;
        resample();
        std::cout << ")";
    }
}

int main()
{
    livenessTest();
    return 0;
}
