// Test for Resample.hxx
// basic compile might look as follows:
// g++ -o ResampleTest ResampleTest.cc -I/home/maciek/opt/include -I/home/rayg/opt/boost/include -I../Common -L/home/maciek/opt/lib -lfftw3

#include <iostream>
#include <boost/array.hpp>
#include "Resample.hxx"
#include "Physics.h"
#include "CdfFile.hxx"

using namespace std;
using namespace gips;

//enum {SPECTRUM_SIZE = 1025, UPSAMPLED_SIZE = 262144};
const double LWN = 9398.49609375;
const unsigned DECIMATION = 8;

const unsigned SPC_SIZE=1024;
const unsigned RESAMPLE_UPSAMPLED_SIZE = 262144;
typedef boost::array< double, SPC_SIZE > RealSpectrum; 
typedef boost::array< std::complex<double>, SPC_SIZE > CplxSpectrum; 
typedef SpectralResampling<RealSpectrum, RealSpectrum, RESAMPLE_UPSAMPLED_SIZE> Resample;
typedef CdfFile<RealSpectrum, CplxSpectrum> Cdf;

/*
typedef boost::array<double, SPECTRUM_SIZE> DV;
typedef SpectralResampling<DV, DV, UPSAMPLED_SIZE> SpectralResamplingDV;
void livenessTest()
{
    SpectralResamplingDV::settings_t settings;
    settings.wavenumberScale = WavenumberScale<double, SPECTRUM_SIZE>(10.0, 100.0);

    SpectralResamplingDV::ancillary_t ancil;
    ancil.laserWavenumberRatio = 0.999;
    
    DV testin, testout;
    fill(testin.begin(), testin.end(), 1.0);
    fill(testout.begin(), testout.end(), 0.0);

    SpectralResamplingDV resampler(settings, SpectralResamplingDV::Ports(&ancil, &testin, &testout));
    resampler();
}
*/

void test()
{
    RealSpectrum in;
    fill(in.begin(), in.end(), 1.0);

    RealSpectrum out;

    Resample::ancillary_t anc;
    const double bb(0.15243531037903);
    anc.laserWavenumberRatio = 1 + (1.0 + bb) / 2000000;

    physics::WnumScale<double, SPC_SIZE> wavenumberScale(physics::wnums_t::fromLaserWnum(SPC_SIZE, LWN, DECIMATION ));

    Resample::settings_t settings(wavenumberScale);
    Resample::Ports ports(&anc, &in, &out);
    Resample resample(settings, ports);
    resample();

    Cdf f("test.nc", Cdf::Replace);
    f.addDim("I", 2);
    f.addDim("J", 2);
    f.addDim("SPC_SIZE", SPC_SIZE);
    f.addDoubleVar("signalReal", "I", "J", "SPC_SIZE");
    f.writeRec("signalReal", in, 0, 0);
    f.writeRec("signalReal", out, 1, 0);
    f.close();
}


int main()
{
//    livenessTest();
    test();
    return 0;
}
